
package com.reactlibrary;

import android.annotation.SuppressLint;
import android.app.Notification;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Build;
import android.util.Log;

import androidx.annotation.Nullable;

import com.baidu.location.BDAbstractLocationListener;
import com.baidu.location.BDLocation;
import com.baidu.location.LocationClient;
import com.baidu.location.LocationClientOption;
import com.baidu.mapapi.CoordType;
import com.baidu.mapapi.SDKInitializer;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableMap;
import com.facebook.react.modules.core.DeviceEventManagerModule;
import com.reactlibrary.activity.BaiduMapActivity;
import com.reactlibrary.listener.UpdateLocationListener;
import com.reactlibrary.util.NotificationUtils;
import com.reactlibrary.util.SharedUtil;

public class RNReactNativeBaiduMapLocationModule extends ReactContextBaseJavaModule {
  private static final String TAG = "RNReactNativeBaiduMapLocationModule";
  private static final String TYPE_FAIL_LOCATION = "typeFailLocation";
  private static final String TYPE_UPDATE_LOCATION = "typeUpdateLocation";
  private final ReactApplicationContext reactContext;

  private MyLocationListener listener = new MyLocationListener();
  private LocationClient mLocationClient = null;
  private NotificationUtils mNotificationUtils;
  private Notification notification;
  private String deviceToken;
  private UpdateLocationListener updateLocationListener;
  private ReadableMap openActivityOptionMap;

  public RNReactNativeBaiduMapLocationModule(ReactApplicationContext reactContext) {
    super(reactContext);
    this.reactContext = reactContext;
    //在使用SDK各组件之前初始化context信息，传入ApplicationContext
    SDKInitializer.initialize(reactContext.getApplicationContext());
    //自4.3.0起，百度地图SDK所有接口均支持百度坐标和国测局坐标，用此方法设置您使用的坐标类型.
    //包括BD09LL和GCJ02两种坐标，默认是BD09LL坐标。
    SDKInitializer.setCoordType(CoordType.BD09LL);
  }

  @Override
  public String getName() {
    return "RNReactNativeBaiduMapLocation";
  }

  /**
   * 初始化定位参数配置
   */
  @ReactMethod
  public void initLocationOption(ReadableMap optionMap) {
    //定位服务的客户端。宿主程序在客户端声明此类，并调用，目前只支持在主线程中启动
    if (mLocationClient == null) {
      mLocationClient = new LocationClient(reactContext.getApplicationContext());     //声明LocationClient类
    }
    mLocationClient.registerLocationListener(listener);
    createNotification();

    //声明LocationClient类实例并配置定位参数
    LocationClientOption locationOption = new LocationClientOption();
    //可选，默认高精度，设置定位模式，高精度，低功耗，仅设备
    locationOption.setLocationMode(LocationClientOption.LocationMode.Hight_Accuracy);
    //可选，默认gcj02，设置返回的定位结果坐标系，如果配合百度地图使用，建议设置为bd09ll;
    String coorType = optionMap.getString("coorType");
    if (coorType != null) locationOption.setCoorType(coorType);
    //可选，默认0，即仅定位一次，设置发起连续定位请求的间隔需要大于等于1000ms才是有效的
    locationOption.setScanSpan(optionMap.getInt("scanSpan"));
    //可选，设置是否需要设备方向结果
    locationOption.setNeedDeviceDirect(false);
    //可选，默认false，设置是否当gps有效时按照1S1次频率输出GPS结果
    locationOption.setLocationNotify(true);
    //可选，默认true，定位SDK内部是一个SERVICE，并放到了独立进程，设置是否在stop的时候杀死这个进程，默认不杀死
    locationOption.setIgnoreKillProcess(optionMap.getBoolean("ignoreKillProcess"));
    //可选，默认false，设置是否需要POI结果，可以在BDLocation.getPoiList里得到
    locationOption.setIsNeedLocationPoiList(true);
    //可选，默认false，设置是否收集CRASH信息，默认收集
    locationOption.SetIgnoreCacheException(false);
    //可选，默认false，设置是否开启Gps定位
    locationOption.setOpenGps(optionMap.getBoolean("openGps"));
    //可选，默认false，设置定位时是否需要海拔信息，默认不需要，除基础定位版本都可用
    locationOption.setIsNeedAltitude(false);
    //设置打开自动回调位置模式，该开关打开后，期间只要定位SDK检测到位置变化就会主动回调给开发者，该模式下开发者无需再关心定位间隔是多少，定位SDK本身发现位置变化就会及时回调给开发者
    locationOption.setOpenAutoNotifyMode();
    //设置打开自动回调位置模式，该开关打开后，期间只要定位SDK检测到位置变化就会主动回调给开发者
    locationOption.setOpenAutoNotifyMode(3000,1, LocationClientOption.LOC_SENSITIVITY_HIGHT);
    //需将配置好的LocationClientOption对象，通过setLocOption方法传递给LocationClient对象使用
    mLocationClient.setLocOption(locationOption);
    //开始定位
    mLocationClient.start();
  }

  @ReactMethod
  public void openBaiduMap(ReadableMap optionMap) {
    openActivityOptionMap = optionMap;
    Intent intent = new Intent(reactContext, BaiduMapActivity.class);
    reactContext.startActivity(intent);
    Log.i(TAG, getCurrentActivity().toString());
  }

  @ReactMethod
  public void stopLocation() {
    if (mLocationClient != null) {
      mLocationClient.stop();
    }
  }

  public void registerUpdateLocationListener(UpdateLocationListener updateLocationListener) {
    this.updateLocationListener = updateLocationListener;
  }

  public void unregisterUpdateLocationListener() {
    this.updateLocationListener = null;
  }

  public ReadableMap getOpenActivityOptionMap() {
    return openActivityOptionMap;
  }

  private void sendSuccessEvent(BDLocation location) {
    WritableMap map = Arguments.createMap();
    map.putString("locationType", "baidu");
    map.putString("planeSrType", "bj54");
    map.putDouble("latitude", location.getLatitude());
    map.putDouble("longitude", location.getLongitude());
    map.putDouble("x", location.getLongitude());
    map.putDouble("y", location.getLatitude());
    map.putDouble("radius", location.getRadius());
    map.putDouble("altitude", location.getAltitude());
    map.putDouble("direction", location.getDirection());
    map.putDouble("speed", location.getSpeed());
    map.putString("address", location.getAddrStr());
    map.putString("time", location.getTime());
    map.putString("deviceToken", getDeviceToken());
    sendEvent(TYPE_UPDATE_LOCATION, map);
  }

  private void sendFailureEvent(BDLocation location) {
    WritableMap map = Arguments.createMap();
    if (location != null) {
      map.putInt("locType", location.getLocType());
      map.putString("locDescribe", location.getLocationDescribe());
    }
    sendEvent(TYPE_FAIL_LOCATION, map);
  }

  @SuppressLint("LongLogTag")
  public void sendEvent(String eventName, @Nullable WritableMap params) {
    //此处需要添加hasActiveCatalystInstance，否则可能造成崩溃
    //问题解决参考: https://github.com/walmartreact/react-native-orientation-listener/issues/8
    if (reactContext.hasActiveCatalystInstance()) {
      Log.i(TAG, "hasActiveCatalystInstance");
      reactContext.getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter.class)
              .emit(eventName, params);
    } else {
      Log.i(TAG, "not hasActiveCatalystInstance");
    }
  }

  private void createNotification() {
    if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
      mNotificationUtils = new NotificationUtils(reactContext.getApplicationContext());
      Notification.Builder builder2 = mNotificationUtils.getAndroidChannelNotification
              ("适配android 8限制后台定位功能", "正在后台定位");
      notification = builder2.build();
    } else {
      //获取一个Notification构造器
      Notification.Builder builder = new Notification.Builder(reactContext.getApplicationContext());
      builder.setContentTitle("后台定位功能") // 设置下拉列表里的标题
              .setSmallIcon(R.drawable.ic_launcher) // 设置状态栏内的小图标
              .setContentText("正在后台定位") // 设置上下文内容
              .setWhen(System.currentTimeMillis()); // 设置该通知发生的时间
      notification = builder.build(); // 获取构建好的Notification
    }
  }

  private String getDeviceToken() {
    if (deviceToken != null) return deviceToken;
    SharedPreferences sp = getReactApplicationContext().getSharedPreferences(SharedUtil.FILE_SESSION, Context.MODE_PRIVATE);
    deviceToken = sp.getString(SharedUtil.KEY_DEVICE_TOKEN, null);
    return deviceToken;
  }

  /**
   * 实现定位回调
   */
  private class MyLocationListener extends BDAbstractLocationListener {
    @Override
    public void onReceiveLocation(BDLocation location){
      if (location != null && location.getLocType() != BDLocation.TypeServerError) {
        if (updateLocationListener != null) updateLocationListener.updateLocation(location);
        sendSuccessEvent(location);
      } else {
        sendFailureEvent(location);
      }
    }
  }

}